#ifndef __M_STEERG_H
#define __M_STEERG_H

// File m_steerg.h
// (C) 1993,1994 by Heiko Damerow MPG AG "Rntgenbeugung"

#include		"m_data.h"
#include		"m_xscan.h"

#define			nMaxArg 		4

// Commands
typedef enum {	GotoPeak = 1,GotoIntensity,ChooseAxis,GetHWB,SetWidth,
								AreaScan,Scan,ControlFlank,ShowHWB,SaveData,ChooseDevice,
								Calculate,LoadPoint,ShowValue,MoveToPoint,SetFileName,
								Inquire,GotoLine,Stop,SetupScan,SetupAreaScan
								} TCmdId;

// Command-Parameter
typedef enum {	LargeSide = 1,SmallSide,ToSmallerAngle,ToLargerAngle,
								Equidistant,Interpolation,Reflection,LastGoal,Standard,
								DecreaseWidth,ThisDFPos,BackMove,MaximizeTilt,
								MaximizeCollimator,MaximizeGradient,DynamicWidth,Result,
								StaticStepWidth,IncreasePeak,Relative,AreaScanResult,
								AbsorberUsed,Hwb,Difference,ScanResult,
								Middle,Array,Start,Peak,Min,Max,List,Argument,Opposite,
								ForAreaScan,ForScan
								} TCParam;

// Steuer-Codes fr Commands
typedef enum {	CReady = 1,CHalted,CMeasure,CRecall,CReadyStep,
								CFirstStep,CControlStep,CStop,CFailure
								} TCCode;

// Macro Bezeichner

typedef enum {	InquireHwb = 1,BatchMacro,MiddleOfValley,SearchReflection,
								Test,AzimutalJustify,SetupTopography,ScanJob,AreaScanJob
								} TMacroId;

typedef float JPoint[5];

typedef struct
	{
	float			fStartPoint,fSpeed,fExposureTime;
	int				nNumberPoints;
	int				nMotorId;
	int				nDeviceId;
	BOOL			bFastScanActive;
	} TCScanParam;

typedef struct
	{
	TCmdId		Id;
	TCParam		P1;
	TCParam		P2;
	char			P3[51];
	} TCmdTag;

//*************************************************************
class	__rtti TCmd
	{
	friend  				class TSteering;

	public: 				TCmd(TCmdTag);
	virtual					TCmd::~TCmd(void);
	TCCode  				DoAction(void);
	virtual TCCode  FirstStep(void) 			{return CReady;};
	virtual TCCode  WakeUp(void);
	virtual BOOL	 	DeviceRequest(void);
	virtual BOOL	 	GetShowData(LPSTR)		{return TRUE;};
	virtual TCCode  ControlStep(void)			{return CReady;};
	virtual TCCode  ReadyStep(void)				{return CReady;};
	virtual TCCode  Ready(void)						{return CReady;};
	BOOL		 				IsPositionValid(void)	{return bPositionValid;};
	BOOL						IsReady(void)					{return (eStep == CReady);};
	TCmdId					GetId(void) {return eCmdId;};
	BOOL						StartMove(const int,double);
	virtual LPSTR		GetFailure(void);

	protected:
	TCCode					eStep;
	BOOL            bNoMeasure;   // normaly starts the device after moving
	BOOL*           bMoveOk;
	int             nFailureId;

	private:
	TCmdId					eCmdId;
	BOOL						bPositionValid;
	};

typedef struct
	{
	TMacroId 		Id;
	int					Length;
	BOOL				bIsReady;
	TCmdTag*		CmdList;
	char				Name[21];
	char				FileName[21];
	} TMacroTag;

class TSteering
	{
	friend TIMECALLBACK	RecallSteering;
	friend LRESULT CALLBACK FrameWndProc(HWND,UINT,WPARAM,LPARAM);

	friend class				TCmd;
	friend class				TMacroExecute;

	public:         TSteering(void);
	BOOL						Initialize(const HWND,const int,const int);
	BOOL            StartUp(const HWND,const int,const int);
	BOOL						Reset(void);
	void            Visualising(int,int,int,int,int,int);
	int             StartCmdExecution(TCmdTag);
	int             StartCmdExecution(TCmdId,int,int,LPSTR,HWND);
	int             StartMacroExecution(TMacroTag*,HWND);
	BOOL	 					DeviceRequest(void);
	void            SetNorm(float norm) {fIntensityNorm = norm;};
	BOOL						ToggleInterrupt(void);
	int 						CallMacro(TMacroTag*);
	float						GetIntensity(void);
	BOOL						IsResponding(void)		{return bSendReport;};
	void						SetResponse(BOOL sr)	{bSendReport = sr;};
	double					GetDistance(void);
	void						NotifyCmdReady(void);
	void						NotifyMacroReady(void);
	HWND 						GetControlWnd(void) 	{return hControlWnd;};
	BOOL						IsPositionValid(void);
	void						SetNoiseLevel(float nl) {fNoiseLevel = nl;};
	TMacroTag*			GetMacroByNumber(int);
	TMacroTag*			GetMacroById(TMacroId);
	TMacroTag* 			GetMacroByName(LPSTR);
	BOOL						LoadMacro(LPSTR,LPSTR);
	BOOL						DeleteMacro(TMacroTag*);
	BOOL						LoadMacroByUser(void);
	float						GetSigma(void) 					{return Device->GetSigma();};
	float						GetNoiseLevel(void) 		{return fNoiseLevel;};

	double					dPeakPoint,dStartPoint;
	float						fPeakIntensity,fStartIntensity,fHWB;
	double					dCalcResult,dCalcArg[nMaxArg];
	double        	dGoalDistance;
	float		      	fGoalIntensity;
	BOOL						bInquireResult;

	protected:
	void            SendReport(void);
	HWND						hReportList;
	BOOL            b_id_Angle;
	BOOL            b_id_PeakIntensity;
	BOOL            b_id_Intensity;
	BOOL            b_id_HalfWidth;
	BOOL            b_id_Report;
	BOOL            bSendReport;
	//*** Zustands-Flags ****************
	BOOL            bInterrupted;
	TDevice*        Device;
	int							nMotor;
	int							nFailureId;
	TMacroTag				aMacroList[20];
	int							nMacroNumber;
	void						WakeUp(void);

	private:
	char						MessageBuffer[MaxString];
	BOOL						ParsingMacroId(TMacroTag&,LPSTR);
	BOOL 						ParsingCmd(TCmdTag&,LPSTR,LPSTR,LPSTR,LPSTR);
	TCParam 				ParsingCmdParam(LPSTR param);
	BOOL 						GetFileLine(FILE*,LPSTR,int);
	BOOL 						ReadyReaction(void);
	int		  				TimerRequest(void);
	int             ExecuteNextCmd(void);
	BOOL			      StartTimer(int);
	BOOL		        StopTimer(void);
	BOOL						bTimerActive;
	BOOL            IsMacroCalled(void);
	int             ReturnMacroCall(void);
	float						fIncomingIntensity;
	TMacroTag* 			TheMacro;
	TCmdTag*		    TheCmdList;
	TCmd*				    TheCmd;
	int							nCmdIdx;
	BOOL     				bSteeringActive;
	float           fIntensityNorm;
	float           fNoiseLevel;
	HWND						hControlWnd;
	};

//******************************************************************************
class	__rtti TLoadPointCmd : public TCmd
	{
	public: 			TLoadPointCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);
	};

class	__rtti TSetFileNameCmd : public TCmd
	{
	public: 			TSetFileNameCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);

	private:
	TScan*				ScanW;
	TAreaScan* 		AreaScanW;
	};

class	__rtti TSaveDataCmd : public TCmd
	{
	public: 			TSaveDataCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);

	private:
	TScan*				ScanW;
	TAreaScan* 		AreaScanW;
	};

class	__rtti TChooseAxisCmd : public TCmd
	{
	public: 			TChooseAxisCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);
	};

class	__rtti TSetupScanCmd : public TCmd
	{
	public: 			TSetupScanCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);
	private:
	TScan*				ScanW;
	};

class	__rtti TChooseDeviceCmd : public TCmd
	{
	public: 			TChooseDeviceCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);
	};

class	__rtti TSetWidthCmd : public TCmd
	{
	public: 			TSetWidthCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);
	};

class	__rtti TInquireCmd : public TCmd
	{
	public: 			TInquireCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);
	};

class	__rtti TShowValueCmd : public TCmd
	{
	public: 			TShowValueCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);
	};

class	__rtti TCalculateCmd : public TCmd
	{
	public: 			TCalculateCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);
	};

class	__rtti TMoveToPointCmd : public TCmd
	{
	public: 			TMoveToPointCmd(TCmdTag);
	BOOL	 				GetShowData(LPSTR);
	TCCode  			ReadyStep(void);
	};

class	__rtti TScanCmd  : public TCmd
	{
	public: 			TScanCmd(TCmdTag);
	TCCode  			FirstStep(void);
	TCCode  			ControlStep(void);
	TCCode  			ReadyStep(void);
	BOOL	 				GetShowData(LPSTR);

	private:
	TCParam				eCaller;
	TxScanType		eScanType;
	TScan*				ScanW;
	TAreaScan* 		AreaScanW;
	int						nPointIdx,nPointNumber;
	int						nMotor,nTheta,nAbsorber;
	double				dArgumentMin,dArgumentMax,dArgumentWidth;
	double				dThetaMin,dThetaMax,dThetaWidth;
	double				dStartPointTheta,dStartPointAbsorber;
	BOOL					bOmega2Theta;
	BOOL					bAbsorberUsed;
	double				dMoveRelation;
	double				dArgument;
	float					fDriveInIntensity;
	float					fDriveOutIntensity;
	float					fIntensityRelation;
	BOOL 					bAbsorberActivated;
	BOOL 					bAbsorberMovedIn;
	BOOL					bDynamicStepWidth;
	int						nStepScaleLevels;
	float*				Scaler;
	};

class	__rtti TGotoPeakCmd  : public TCmd
	{
	public: 			TGotoPeakCmd(TCmdTag);
								~TGotoPeakCmd();
	TCCode  			FirstStep(void);
	TCCode  			ControlStep(void);
	TCCode  			ReadyStep(void);
	BOOL	 				GetShowData(LPSTR);

	private:
	int						nMotor;
	BOOL					bSmallAngleSide;
	BOOL					bFlankDetected;
	BOOL					bDynamicStepWidth;
	BOOL				  bDirectionChanged;
	float					fSearchIntensity;
	TCParam				nReadyAction;
	int						nLifo;
	double*				dDistance;
	float*				fIntensity;
	BOOL					bBackMoveActive;
	};

class	__rtti TControlFlankCmd : public TCmd
	{
	public: 			TControlFlankCmd(TCmdTag);
								~TControlFlankCmd();
	TCCode  			FirstStep(void);
	TCCode  			ControlStep(void);
	TCCode  			ReadyStep(void);
	BOOL	 				GetShowData(LPSTR);

	private:
	int						nMotor;
	float					fControlRange;
	float					fStartIntensity,fSearchIntensity;
	float					fMinIntensity,fMaxIntensity,fIntensityRange;
	BOOL					bSmallAngleSide;
	int						nLifo;
	double*				dDistance;
	float*				fIntensity;
	};

class	__rtti TGotoIntensityCmd  : public TCmd
	{
	public: 			TGotoIntensityCmd(TCmdTag);
								~TGotoIntensityCmd();
	TCCode  			FirstStep(void);
	TCCode  			ControlStep(void);
	TCCode  			ReadyStep(void);
	BOOL	 				GetShowData(LPSTR);

	private:
	int						nMotor;
	BOOL					bSmallAngleSide;
	float					fSearchIntensity;
	int						nLifo;
	double*				dDistance;
	float*				fIntensity;
	TCParam				nReadyAction;
	};

class	__rtti TAreaScanCmd  : public TCmd
	{
	public: 			TAreaScanCmd	(TCmdTag);
	TCCode  			FirstStep			(void);
	TCCode  			ControlStep		(void);
	TCCode  			ReadyStep			(void);
	BOOL	 				GetShowData		(LPSTR);

	private:
	TAreaScan*		AreaScanW;
	int						nOmega,nTheta;
	double				dThetaWindow,dThetaWidth;
	double				dOmegaMin,dOmegaMax,dOmegaWidth;
	double				dStartPointTheta,dStartPointOmega;
	double				dMoveRelation;
	double				dOmega,dTheta;
	int						nMaxThetaPoint,nThetaPoint;
	int						nMaxOmegaPoint,nOmegaPoint;
	};
//******************************************************************************
//******************************************************************************
class TAdjustmentWindow : public TPlotData
	{
	public: 				TAdjustmentWindow(void);
									~TAdjustmentWindow();
	protected:
	BOOL            CanClose(void);
	void            Create(HWND);
	void 						InitializeTask(WPARAM,LPARAM);
	void            DrawMenu(void);
	void            rButtonDown(LPARAM);
	void            StopMeasurement(void);
	LPCSTR          ClassName() { return "Adjustment";};
	LPCSTR          GetCharacteristic() { return "Adjustment";};
	void            CounterSetRequest(LPARAM);
	void            SteeringReady(LPARAM);

	private:
	BOOL					  bContinuousScanActive;
	TCurve*					AdjData;
	BOOL            bMacroRunning;

	BOOL            SetTitle(void);
	};

class TEditWindow : public TMDIWindow
	{
	public:         TEditWindow(LPSTR aFilename);
									~TEditWindow();
	char            FileName[MAXPATH];
	BOOL            bIsNewFile;
	BOOL            bFileChanged;

	void            Create(HWND);
	BOOL            SetTitle(void);
	BOOL            IsModified(void) {return bFileChanged;};
	BOOL            CanClear(void);
	BOOL            NewFile(void);
	BOOL            SaveFile(int);
	BOOL            ReadFile(void);
	void            rButtonDown(LONG lParam);
	void            ClearModify() {};
	HCURSOR         GetCursor(void)
										{ return LoadCursor(NULL,IDC_IBEAM);};
	void            Size(HWND,WPARAM,LPARAM);
	void            Paint(HDC,PAINTSTRUCT *);
	void            SetFocus(void);
	void            ClearWindow(void){};
	void            DoSearch(void){};
	LPCSTR          ClassName(void) { return "Editor";};
	LPCSTR          GetCharacteristic(void) { return "Edit";};
	};

class TSetAdjustmentParam : public TModalDlg
	{
	public:         TSetAdjustmentParam();

	protected:
	float           NoiseLevel;
	float           SearchFailure;

	private:
	BOOL            CanClose(void);
	void            SetParameter(void);
	void            GetParameter(void);
	void            Dlg_OnCommand(HWND,int,HWND,UINT);
	BOOL            Dlg_OnInit(HWND,HWND,LPARAM);
	BOOL            LeaveDialog(void);
	};

class TAdjustmentParameter
	{
	friend class		TSetAdjustmentParam;
	friend class		TAdjustmentExecute;

	public:         TAdjustmentParameter();

	protected:
	float           fNoiseLevel;
	float           fFailure;
	double          dAngleRange;
	};

class TMacroExecute : public TModelessDlg
	{
	public:   			TMacroExecute(void);

	private:
	TMacroTag* 			TheMacro;
	TDevice*				Device;
	HWND            hMacroList;
	BOOL            Dlg_OnInit(HWND,HWND,LPARAM);
	void            Dlg_OnCommand(HWND,int,HWND,UINT);
	BOOL            CanClose(void);
	BOOL            LeaveDialog(void);
	int             MessageLine;
	BOOL            bInterrupted;
	BOOL            bActionCaused;
	int             SetupTask;
	BOOL            bCheckMacro;
	};

class TAdjustmentExecute : public TModalDlg
	{
	public:   			TAdjustmentExecute(int);

	private:
	HWND            hMotorList;
	BOOL            Dlg_OnInit(HWND,HWND,LPARAM);
	TMacroTag*   		GetNextTask(void);
	void            Dlg_OnCommand(HWND,int,HWND,UINT);
	BOOL            CanClose(void);
	void            GetParameter(void);
	BOOL            LeaveDialog(void);
	int							nMotor;
	TDevice*				Device;
	int             MessageLine;
	BOOL            bJustifyActive;
	BOOL            bInterrupt;
	BOOL            bActionCaused;
	int             SetupTask;
	BOOL            bReflectionSearch;
	BOOL            bAzimutalAdjustment;
	BOOL            bFinalAdjustment;
	BOOL            bGetPeakProperties;
	BOOL            bSearchStartPosition;
	BOOL            bCheckMacro;
	};
#endif   
// __M_STEERG_H
// __LastLine__
